import { UnknownSymbolError } from '@/utils';
import type { DenotableValue } from '.';

export class Environment {
  private scope: Map<string, DenotableValue>;
  private parent: Environment | null;

  constructor(parent: Environment | null = null) {
    this.parent = parent;
    this.scope = new Map();
  }

  public set(name: string, value: DenotableValue) {
    this.scope.set(name, value);
  }

  public get(name: string): DenotableValue {
    if (this.scope.has(name)) {
      return this.scope.get(name);
    }

    if (this.parent) {
      return this.parent.get(name);
    }

    throw new UnknownSymbolError(`Undefined variable: ${name}`);
  }

  public has(name: string): boolean {
    if (this.scope.has(name)) {
      return true;
    }

    if (this.parent) {
      return this.parent.has(name);
    }

    return false;
  }

  public createChild(): Environment {
    return new Environment(this);
  }
}
